#include <cstdlib>
#include <iostream>
#include <cmath>
#include <vector>
#include "game.h"
using namespace std;

// program entry point
int main(int argc, char** argv){
	//cout << "MAIN\n";
	Game* g;

	g = new Game(argc, argv);
	g->init();
	g->run();

	//cout << "END_MAIN\n";
	return g->return_code;
} // main

// set config flags from arguments
void Game::process_args(){
	//cout << "PROCESS_ARGS\n";

	// TODO: do something with these

} // process_args

// initialization manager
void Game::init(){
	//cout << "INIT\n";

	init_rules();
	init_io();
	init_gfx();

	init_snd();
	init_physics();

	//cout << "END_INIT\n";
} // init

// game specific initializations
void Game::init_rules(){
	//cout << "INIT_RULES\n";

	cout << "Keyboard Controls:\n"
		 << " 1  -  drop a full box of spheres\n"
		 << " 2  -  drop an empty box\n"
		 << " 3  -  drop a cylinder\n"
		 << " 4  -  drop a cloth\n"
		 << " 5  -  create a static sphere\n"
		 << " 6  -  drop a full box of boxes\n"
		 << " 0  -  drop a bean\n"
		 << " s  -  show(on/off) reflections\n"
		 << " j  -  show(on/off) joints\n"
		 << " r  -  reset\n"
		 << " q  -  squeeze (for the tool)\n"
		 << " e  -  pull apart (for the tool)\n";

	cout << "\nMouse Controls: (for the tool)\n"
		 << " Left Click       -  fire a ball\n"
		 << " Rigth Click      -  fire a cylinder\n"
		 << " Scroll Up        -  ascend\n"
		 << " Scroll Down      -  descend\n"
		 << " Moving the Mouse -  move the tool\n";

} // init_rules

// setup SDL systems
void Game::init_io(){
	//cout << "INIT_IO\n";

	// misc setup
	SDL_Init(SDL_INIT_VIDEO|SDL_INIT_AUDIO);
    atexit(SDL_Quit); // register SDL Parachute for crashes
	SDL_WM_SetCaption("Simulations", NULL); // give our window a name (default icon)

	// NOTE: SDL_Init can be called with SDL_RESIZABLE to allow the user to
	// resize the window.  However, this would require resetting projection
	// and such whenever it happened. KISS.

	// set opengl attributes (is this necesary on all systems?)
    SDL_GL_SetAttribute(SDL_GL_RED_SIZE,        5);
    SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE,      5);
    SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE,       5);
#ifdef __APPLE__
    SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE,      32);
#else
    SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE,      16);
#endif
    SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER,    1);

	// create a framebuffer
	const SDL_VideoInfo* vidinfo = SDL_GetVideoInfo();
	SDL_Surface* screen = NULL;
	Uint32 fs = show_fullscreen?SDL_FULLSCREEN:0;
	screen = SDL_SetVideoMode(win_width, win_height, vidinfo->vfmt->BitsPerPixel, SDL_OPENGL|fs);
	if (!screen){
		cerr << "Couldn't set video mode!" << endl;
		cerr << SDL_GetError() << endl;
		exit(1);
	}

	SDL_ShowCursor(0);
	SDL_WM_GrabInput(SDL_GRAB_ON);

	//cout << "END_INIT_IO\n";
} // init_io

// setup GL and load visual assets
void Game::init_gfx(){
	//cout << "INIT_GFX\n";
	// viewport
    glViewport(0, 0, win_width, win_height);

	// matricies
	glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(45.0, ar(), 10.0, 1000.0);

	glMatrixMode(GL_MODELVIEW);
	// clear color
	glClearColor(0.0, 0.0, 0.0, 0.0);

	// misc GL states
	glEnable(GL_NORMALIZE);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable(GL_COLOR_MATERIAL);

	const float ambient[] = {0.05, 0.05, 0.05};
	glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
	const float specular[] = {0.75, 0.75, 0.75};

	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
	glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 50.0);

	glShadeModel(GL_SMOOTH);
	
	// this would be a good place to load any fancy geometry assset required 

	quad = gluNewQuadric();

	//cout << "END_INIT_GFX\n";
} // init_gfx

// setup audio device and load audio assets
void Game::init_snd(){
	//cout << "INIT_SND\n";
	// setup audio mixer
    Mix_OpenAudio(44100, AUDIO_S16, 2, 4096); // common settings
    // only if needed: Mix_AllocateChannels(128);

	//cout << "END_INIT_SND\n";
} // init_snd

void Game::init_physics(){
	//cout << "INIT_PHYSICS\n";

	// grab time
	ticks = SDL_GetTicks() - 1; // -1 ensures SOME time pass for first frame's motion

	// create containers
	world = dWorldCreate();
	space = dHashSpaceCreate(0);
	contact_jgroup = dJointGroupCreate(0);

	//dWorldSetERP(world, 0.5);

	// gravity
	dWorldSetGravity(world, 0, 0, -9.81);

	init_tool();

	//////////
	//PLANES//
	//////////

	// infinite ground plane (has no body!)
	dCreatePlane(space, 0.0, 0.0, 1.0, 0.0);
	
	//cout << "END_INIT_PHYSICS\n";
} // init_physics

// a full box made out of spheres
void Game::init_full_box(){
	//cout << "INIT_FULL_BOX\n";

	vector<dBodyID> v_full_box;

	dMass m;
	dReal total_m = 0.0000001;
	full_boxes_sphere_r = 0.25;
	double d = 2 * full_boxes_sphere_r;
	full_boxes_sphere_slices = 4;
	full_boxes_sphere_stacks = 4;
	dMassSetSphereTotal(&m, total_m, full_boxes_sphere_r);

	int n = 9;
	const dReal center_x = mx;
	const dReal center_y = my;
	const dReal start_x = ((n-1)/2)*d + center_x;
	const dReal start_y = center_y - ((n-1)/2)*d;
	const dReal start_z = 40.0;

	for (int k = 0; k < n; k++){ // each height
		for (int i = 0; i < n; i++){ // each row
			for (int j = 0; j < n; j++){ // each column
				dBodyID b = dBodyCreate(world);
				dGeomID g = dCreateSphere(space, full_boxes_sphere_r);
				dBodySetMass(b, &m);
				dGeomSetBody(g, b);
				dBodySetPosition(b, start_x - i * d, start_y + j * d, start_z + k * d);
				dBodySetData(b, NULL); // set the user pointer to something that helps us to identify the body
				dBodySetAutoDisableFlag (b, 1);
				v_full_box.push_back(b);
			} // for each column
		} // for each row
	} // for height

	v_full_boxes.push_back(v_full_box);

	//cout << "END_INIT_FULL_BOX\n";
} // init_full_box

// a full box made out of spheres
void Game::init_full_box2(){
	//cout << "INIT_FULL_BOX2\n";

	vector<dBodyID> v_full_box2;

	dMass m;
	dReal total_m = 0.0000001;
	full_boxes2_box_lx = 2.0;
	full_boxes2_box_ly = 2.0;
	full_boxes2_box_lz = 2.0;
	dMassSetBoxTotal (&m, total_m, full_boxes2_box_lx, full_boxes2_box_ly, full_boxes2_box_lz);

	int n = 9;
	const dReal center_x = mx;
	const dReal center_y = my;
	const dReal start_x = ((n-1)/2)*full_boxes2_box_lx + center_x;
	const dReal start_y = center_y - ((n-1)/2)*full_boxes2_box_lx;
	const dReal start_z = 60.0;

	for (int k = 0; k < n; k++){ // each height
		for (int i = 0; i < n; i++){ // each row
			for (int j = 0; j < n; j++){ // each column
				dBodyID b = dBodyCreate(world);
				dGeomID g = dCreateBox(space, full_boxes2_box_lx, full_boxes2_box_ly, full_boxes2_box_lz);
				dBodySetMass(b, &m);
				dGeomSetBody(g, b);
				dBodySetPosition(b, start_x - i * (full_boxes2_box_lx+0.1), start_y + j * (full_boxes2_box_lx+0.1), start_z + k * (full_boxes2_box_lz+0.1));
				dBodySetData(b, NULL); // set the user pointer to something that helps us to identify the body
				dBodySetAutoDisableFlag (b, 1);
				v_full_box2.push_back(b);
			} // for each column
		} // for each row
	} // for height

	v_full_boxes2.push_back(v_full_box2);

	//cout << "END_INIT_FULL_BOX\n";
} // init_full_box2

// an empty box
void Game::init_box(){
	//cout << "INIT_BOX\n";
	
	vector<dBodyID> v_box;

	dMass m;
	dReal total_m = 0.5;
	boxes_sphere_r = 0.5;
	double d = 2 * boxes_sphere_r;
	boxes_sphere_slices = 4;
	boxes_sphere_stacks = 4;
	dMassSetSphereTotal(&m, total_m, boxes_sphere_r);

	const int n = 9;
	const dReal center_x = mx;
	const dReal center_y = my;
	const dReal start_x  = ((n-1)/2)*d + center_x; // only good for bottom and top plane
	const dReal start_y  = center_y - ((n-1)/2)*d; // only good for bottom and top plane
	const dReal start_z  = 40.0;

	dReal loStop = 0.0;
	dReal hiStop = 1.0;
	dReal paramBounce = 0.0;
	dReal paramCFM = 0.0;
	dReal paramStopERP = 0.8;
	dReal paramStopCFM = 0.0;

	unsigned int start = 0;
	start = v_box.size();
	// top side
	for (int i = 0; i < n; i++){ // each row
		for (int j = 0; j < n; j++){ // each column
			dBodyID b = dBodyCreate(world);
			dGeomID g = dCreateSphere(space, boxes_sphere_r);
			dBodySetMass(b, &m);
			dGeomSetBody(g, b);
			dBodySetPosition(b, start_x - i * d, start_y + j * d, start_z + (n+1)*d);
			dBodySetData(b, NULL); // set the user pointer to something that helps us to identify the body
			v_box.push_back(b);
	
			if(i) {
				dJointID joint1 = dJointCreateSlider(world, 0);
				dJointAttach(joint1, v_box.at(start+(i-1)*n+j), v_box.at(start+i*n+j));
				dJointSetSliderAxis(joint1, -1, 0, 0);		
				dJointSetSliderParam(joint1, dParamLoStop, loStop);
				dJointSetSliderParam(joint1, dParamHiStop, hiStop);
				dJointSetSliderParam(joint1, dParamBounce, paramBounce);
				dJointSetSliderParam(joint1, dParamCFM, paramCFM);
				dJointSetSliderParam(joint1, dParamStopERP, paramStopERP);
				dJointSetSliderParam(joint1, dParamStopCFM, paramStopCFM);
				v_joints.push_back(joint1);
			}
			if(j) {
				dJointID joint1 = dJointCreateSlider(world, 0);
				dJointAttach(joint1, v_box.at(start+i*n+j-1), v_box.at(start+i*n+j));
				dJointSetSliderAxis(joint1, 0, 1, 0);		
				dJointSetSliderParam(joint1, dParamLoStop, loStop);
				dJointSetSliderParam(joint1, dParamHiStop, hiStop);
				dJointSetSliderParam(joint1, dParamBounce, paramBounce);
				dJointSetSliderParam(joint1, dParamCFM, paramCFM);
				dJointSetSliderParam(joint1, dParamStopERP, paramStopERP);
				dJointSetSliderParam(joint1, dParamStopCFM, paramStopCFM);
				v_joints.push_back(joint1);
			}
		} // for each column
	} // for each row

	// bottom side
	unsigned int start2 = v_box.size();
	for (int i = 0; i < n; i++){ // each row
		for (int j = 0; j < n; j++){ // each column
			dBodyID b = dBodyCreate(world);
			dGeomID g = dCreateSphere(space, boxes_sphere_r);
			dBodySetMass(b, &m);
			dGeomSetBody(g, b);
			dBodySetPosition(b, start_x - i * d, start_y + j * d, start_z);
			dBodySetData(b, NULL); // set the user pointer to something that helps us to identify the body
			v_box.push_back(b);
	
			if(i) {
				dJointID joint1 = dJointCreateSlider(world, 0);
				dJointAttach(joint1, v_box.at(start2+(i-1)*n+j), v_box.at(start2+i*n+j));
				dJointSetSliderAxis(joint1, -1, 0, 0);		
				dJointSetSliderParam(joint1, dParamLoStop, loStop);
				dJointSetSliderParam(joint1, dParamHiStop, hiStop);
				dJointSetSliderParam(joint1, dParamBounce, paramBounce);
				dJointSetSliderParam(joint1, dParamCFM, paramCFM);
				dJointSetSliderParam(joint1, dParamStopERP, paramStopERP);
				dJointSetSliderParam(joint1, dParamStopCFM, paramStopCFM);
				v_joints.push_back(joint1);
			}
			if(j) {
				dJointID joint1 = dJointCreateSlider(world, 0);
				dJointAttach(joint1, v_box.at(start2+i*n+j-1), v_box.at(start2+i*n+j));
				dJointSetSliderAxis(joint1, 0, 1, 0);		
				dJointSetSliderParam(joint1, dParamLoStop, loStop);
				dJointSetSliderParam(joint1, dParamHiStop, hiStop);
				dJointSetSliderParam(joint1, dParamBounce, paramBounce);
				dJointSetSliderParam(joint1, dParamCFM, paramCFM);
				dJointSetSliderParam(joint1, dParamStopERP, paramStopERP);
				dJointSetSliderParam(joint1, dParamStopCFM, paramStopCFM);
				v_joints.push_back(joint1);
			}
		} // for each column
	} // for each row

	// right side
	unsigned int start3 = v_box.size();
	for (int i = 0; i < n+2; i++){ // each row
		for (int j = 0; j < n; j++){ // each column
			dBodyID b = dBodyCreate(world);
			dGeomID g = dCreateSphere(space, boxes_sphere_r);
			dBodySetMass(b, &m);
			dGeomSetBody(g, b);
			dBodySetPosition(b, center_x + ((n-1)/2)*d - j * d, center_y + ((n-1)/2)*d + d, start_z + i * d);
			dBodySetData(b, NULL); // set the user pointer to something that helps us to identify the body
			v_box.push_back(b);
	
			if(i) {
				dJointID joint1 = dJointCreateSlider(world, 0);
				dJointAttach(joint1, v_box.at(start3+(i-1)*n+j), v_box.at(start3+i*n+j));
				dJointSetSliderAxis(joint1, 0, 0, 1);		
				dJointSetSliderParam(joint1, dParamLoStop, loStop);
				dJointSetSliderParam(joint1, dParamHiStop, hiStop);
				dJointSetSliderParam(joint1, dParamBounce, paramBounce);
				dJointSetSliderParam(joint1, dParamCFM, paramCFM);
				dJointSetSliderParam(joint1, dParamStopERP, paramStopERP);
				dJointSetSliderParam(joint1, dParamStopCFM, paramStopCFM);
				v_joints.push_back(joint1);
			}
			if(j) {
				dJointID joint1 = dJointCreateSlider(world, 0);
				dJointAttach(joint1, v_box.at(start3+i*n+j-1), v_box.at(start3+i*n+j));
				dJointSetSliderAxis(joint1, -1, 0, 0);		
				dJointSetSliderParam(joint1, dParamLoStop, loStop);
				dJointSetSliderParam(joint1, dParamHiStop, hiStop);
				dJointSetSliderParam(joint1, dParamBounce, paramBounce);
				dJointSetSliderParam(joint1, dParamCFM, paramCFM);
				dJointSetSliderParam(joint1, dParamStopERP, paramStopERP);
				dJointSetSliderParam(joint1, dParamStopCFM, paramStopCFM);
				v_joints.push_back(joint1);
			}
		} // for each column
	} // for each row

	// left side
	unsigned int start4 = v_box.size();
	for (int i = 0; i < n+2; i++){ // each row
		for (int j = 0; j < n; j++){ // each column
			dBodyID b = dBodyCreate(world);
			dGeomID g = dCreateSphere(space, boxes_sphere_r);
			dBodySetMass(b, &m);
			dGeomSetBody(g, b);
			dBodySetPosition(b, center_x + ((n-1)/2)*d - j * d, center_y - ((n-1)/2)*d - d, start_z + i * d);
			dBodySetData(b, NULL); // set the user pointer to something that helps us to identify the body
			v_box.push_back(b);
	
			if(i) {
				dJointID joint1 = dJointCreateSlider(world, 0);
				dJointAttach(joint1, v_box.at(start4+(i-1)*n+j), v_box.at(start4+i*n+j));
				dJointSetSliderAxis(joint1, 0, 0, 1);		
				dJointSetSliderParam(joint1, dParamLoStop, loStop);
				dJointSetSliderParam(joint1, dParamHiStop, hiStop);
				dJointSetSliderParam(joint1, dParamBounce, paramBounce);
				dJointSetSliderParam(joint1, dParamCFM, paramCFM);
				dJointSetSliderParam(joint1, dParamStopERP, paramStopERP);
				dJointSetSliderParam(joint1, dParamStopCFM, paramStopCFM);
				v_joints.push_back(joint1);
			}
			if(j) {
				dJointID joint1 = dJointCreateSlider(world, 0);
				dJointAttach(joint1, v_box.at(start4+i*n+j-1), v_box.at(start4+i*n+j));
				dJointSetSliderAxis(joint1, -1, 0, 0);		
				dJointSetSliderParam(joint1, dParamLoStop, loStop);
				dJointSetSliderParam(joint1, dParamHiStop, hiStop);
				dJointSetSliderParam(joint1, dParamBounce, paramBounce);
				dJointSetSliderParam(joint1, dParamCFM, paramCFM);
				dJointSetSliderParam(joint1, dParamStopERP, paramStopERP);
				dJointSetSliderParam(joint1, dParamStopCFM, paramStopCFM);
				v_joints.push_back(joint1);
			}
		} // for each column
	} // for each row

	// front side
	unsigned int start5 = v_box.size();
	for (int i = 0; i < n+2; i++){ // each row
		for (int j = 0; j < n+2; j++){ // each column
			dBodyID b = dBodyCreate(world);
			dGeomID g = dCreateSphere(space, boxes_sphere_r);
			dBodySetMass(b, &m);
			dGeomSetBody(g, b);
			dBodySetPosition(b, center_x + ((n-1)/2)*d + d, center_y - ((n-1)/2)*d - d + j * d, start_z + i *d);
			dBodySetData(b, NULL); // set the user pointer to something that helps us to identify the body
			v_box.push_back(b);
	
			if(i) {
				dJointID joint1 = dJointCreateSlider(world, 0);
				dJointAttach(joint1, v_box.at(start5+(i-1)*(n+2)+j), v_box.at(start5+i*(n+2)+j));
				dJointSetSliderAxis(joint1, 0, 0, 1);		
				dJointSetSliderParam(joint1, dParamLoStop, loStop);
				dJointSetSliderParam(joint1, dParamHiStop, hiStop);
				dJointSetSliderParam(joint1, dParamBounce, paramBounce);
				dJointSetSliderParam(joint1, dParamCFM, paramCFM);
				dJointSetSliderParam(joint1, dParamStopERP, paramStopERP);
				dJointSetSliderParam(joint1, dParamStopCFM, paramStopCFM);
				v_joints.push_back(joint1);
			}
			if(j) {
				dJointID joint1 = dJointCreateSlider(world, 0);
				dJointAttach(joint1, v_box.at(start5+i*(n+2)+j-1), v_box.at(start5+i*(n+2)+j));
				dJointSetSliderAxis(joint1, 0, 1, 0);		
				dJointSetSliderParam(joint1, dParamLoStop, loStop);
				dJointSetSliderParam(joint1, dParamHiStop, hiStop);
				dJointSetSliderParam(joint1, dParamBounce, paramBounce);
				dJointSetSliderParam(joint1, dParamCFM, paramCFM);
				dJointSetSliderParam(joint1, dParamStopERP, paramStopERP);
				dJointSetSliderParam(joint1, dParamStopCFM, paramStopCFM);
				v_joints.push_back(joint1);
			}
		} // for each column
	} // for each row

	// back side
	unsigned int start6 = v_box.size();
	for (int i = 0; i < n+2; i++){ // each row
		for (int j = 0; j < n+2; j++){ // each column
			dBodyID b = dBodyCreate(world);
			dGeomID g = dCreateSphere(space, boxes_sphere_r);
			dBodySetMass(b, &m);
			dGeomSetBody(g, b);
			dBodySetPosition(b, center_x - ((n-1)/2)*d - d, center_y - ((n-1)/2)*d - d + j * d, start_z + i *d);
			dBodySetData(b, NULL); // set the user pointer to something that helps us to identify the body
			v_box.push_back(b);
	
			if(i) {
				dJointID joint1 = dJointCreateSlider(world, 0);
				dJointAttach(joint1, v_box.at(start6+(i-1)*(n+2)+j), v_box.at(start6+i*(n+2)+j));
				dJointSetSliderAxis(joint1, 0, 0, 1);		
				dJointSetSliderParam(joint1, dParamLoStop, loStop);
				dJointSetSliderParam(joint1, dParamHiStop, hiStop);
				dJointSetSliderParam(joint1, dParamBounce, paramBounce);
				dJointSetSliderParam(joint1, dParamCFM, paramCFM);
				dJointSetSliderParam(joint1, dParamStopERP, paramStopERP);
				dJointSetSliderParam(joint1, dParamStopCFM, paramStopCFM);
				v_joints.push_back(joint1);
			}
			if(j) {
				dJointID joint1 = dJointCreateSlider(world, 0);
				dJointAttach(joint1, v_box.at(start6+i*(n+2)+j-1), v_box.at(start6+i*(n+2)+j));
				dJointSetSliderAxis(joint1, 0, 1, 0);		
				dJointSetSliderParam(joint1, dParamLoStop, loStop);
				dJointSetSliderParam(joint1, dParamHiStop, hiStop);
				dJointSetSliderParam(joint1, dParamBounce, paramBounce);
				dJointSetSliderParam(joint1, dParamCFM, paramCFM);
				dJointSetSliderParam(joint1, dParamStopERP, paramStopERP);
				dJointSetSliderParam(joint1, dParamStopCFM, paramStopCFM);
				v_joints.push_back(joint1);
			}
		} // for each column
	} // for each row

	//////////
	//JOINTS//
	//////////
	
	int j = 0;
	// connect bottom - front
	for (unsigned int i = start2; i < start2+n; i++){
		dJointID joint = dJointCreateSlider(world, 0);
		dJointAttach(joint, v_box.at(i), v_box.at(start5+1+j));
		dJointSetSliderAxis(joint, 1, 0, 0);
		dJointSetSliderParam(joint, dParamLoStop, loStop);
		dJointSetSliderParam(joint, dParamHiStop, hiStop);
		dJointSetSliderParam(joint, dParamBounce, paramBounce);
		dJointSetSliderParam(joint, dParamCFM, paramCFM);
		dJointSetSliderParam(joint, dParamStopERP, paramStopERP);
		dJointSetSliderParam(joint, dParamStopCFM, paramStopCFM);
		v_joints.push_back(joint);
		j++;
	}
	j = 0;
	// connect bottom - back
	for (unsigned int i = start2+n*(n-1); i < start2+n*(n-1)+n; i++){
		dJointID joint = dJointCreateSlider(world, 0);
		dJointAttach(joint, v_box.at(i), v_box.at(start6+1+j));
		dJointSetSliderAxis(joint, -1, 0, 0);
		dJointSetSliderParam(joint, dParamLoStop, loStop);
		dJointSetSliderParam(joint, dParamHiStop, hiStop);
		dJointSetSliderParam(joint, dParamBounce, paramBounce);
		dJointSetSliderParam(joint, dParamCFM, paramCFM);
		dJointSetSliderParam(joint, dParamStopERP, paramStopERP);
		dJointSetSliderParam(joint, dParamStopCFM, paramStopCFM);
		v_joints.push_back(joint);
		j++;
	}
	j = 0;
	// connect bottom - right
	for (unsigned int i = start2+(n-1); i <= start2+(n-1)+n*(n-1); i += n){
		dJointID joint = dJointCreateSlider(world, 0);
		dJointAttach(joint, v_box.at(i), v_box.at(start3+j));
		dJointSetSliderAxis(joint, 0, 1, 0);
		dJointSetSliderParam(joint, dParamLoStop, loStop);
		dJointSetSliderParam(joint, dParamHiStop, hiStop);
		dJointSetSliderParam(joint, dParamBounce, paramBounce);
		dJointSetSliderParam(joint, dParamCFM, paramCFM);
		dJointSetSliderParam(joint, dParamStopERP, paramStopERP);
		dJointSetSliderParam(joint, dParamStopCFM, paramStopCFM);
		v_joints.push_back(joint);
		j++;
	}
	j = 0;
	// connect bottom - left
	for (unsigned int i = start2; i <= start2+n*(n-1); i += n){
		dJointID joint = dJointCreateSlider(world, 0);
		dJointAttach(joint, v_box.at(i), v_box.at(start4+j));
		dJointSetSliderAxis(joint, 0, -1, 0);
		dJointSetSliderParam(joint, dParamLoStop, loStop);
		dJointSetSliderParam(joint, dParamHiStop, hiStop);
		dJointSetSliderParam(joint, dParamBounce, paramBounce);
		dJointSetSliderParam(joint, dParamCFM, paramCFM);
		dJointSetSliderParam(joint, dParamStopERP, paramStopERP);
		dJointSetSliderParam(joint, dParamStopCFM, paramStopCFM);
		v_joints.push_back(joint);
		j++;
	}
	j = 0;
	// connect top - front
	unsigned int start5_top = start5 + (n+2)*(n+1) + 1;
	for (unsigned int i = start; i < start+n; i++){
		dJointID joint = dJointCreateSlider(world, 0);
		dJointAttach(joint, v_box.at(i), v_box.at(start5_top+j));
		dJointSetSliderAxis(joint, 1, 0, 0);
		dJointSetSliderParam(joint, dParamLoStop, loStop);
		dJointSetSliderParam(joint, dParamHiStop, hiStop);
		dJointSetSliderParam(joint, dParamBounce, paramBounce);
		dJointSetSliderParam(joint, dParamCFM, paramCFM);
		dJointSetSliderParam(joint, dParamStopERP, paramStopERP);
		dJointSetSliderParam(joint, dParamStopCFM, paramStopCFM);
		v_joints.push_back(joint);
		j++;
	}
	j = 0;
	// connect top - back
	unsigned int start6_top = start6 + (n+2)*(n+1) + 1;
	for (unsigned int i = start+n*(n-1); i < start+n*(n-1)+n; i++){
		dJointID joint = dJointCreateSlider(world, 0);
		dJointAttach(joint, v_box.at(i), v_box.at(start6_top+j));
		dJointSetSliderAxis(joint, -1, 0, 0);
		dJointSetSliderParam(joint, dParamLoStop, loStop);
		dJointSetSliderParam(joint, dParamHiStop, hiStop);
		dJointSetSliderParam(joint, dParamBounce, paramBounce);
		dJointSetSliderParam(joint, dParamCFM, paramCFM);
		dJointSetSliderParam(joint, dParamStopERP, paramStopERP);
		dJointSetSliderParam(joint, dParamStopCFM, paramStopCFM);
		v_joints.push_back(joint);
		j++;
	}
	j = 0;
	// connect top - right
	unsigned int start3_top = start3 + n*(n+1);
	for (unsigned int i = start+(n-1); i <= start+(n-1)+n*(n-1); i += n){
		dJointID joint = dJointCreateSlider(world, 0);
		dJointAttach(joint, v_box.at(i), v_box.at(start3_top+j));
		dJointSetSliderAxis(joint, 0, 1, 0);
		dJointSetSliderParam(joint, dParamLoStop, loStop);
		dJointSetSliderParam(joint, dParamHiStop, hiStop);
		dJointSetSliderParam(joint, dParamBounce, paramBounce);
		dJointSetSliderParam(joint, dParamCFM, paramCFM);
		dJointSetSliderParam(joint, dParamStopERP, paramStopERP);
		dJointSetSliderParam(joint, dParamStopCFM, paramStopCFM);
		v_joints.push_back(joint);
		j++;
	}
	j = 0;
	// connect top - left
	unsigned int start4_top = start4 + n*(n+1);
	for (unsigned int i = start; i <= start+n*(n-1); i += n){
		dJointID joint = dJointCreateSlider(world, 0);
		dJointAttach(joint, v_box.at(i), v_box.at(start4_top+j));
		dJointSetSliderAxis(joint, 0, -1, 0);
		dJointSetSliderParam(joint, dParamLoStop, loStop);
		dJointSetSliderParam(joint, dParamHiStop, hiStop);
		dJointSetSliderParam(joint, dParamBounce, paramBounce);
		dJointSetSliderParam(joint, dParamCFM, paramCFM);
		dJointSetSliderParam(joint, dParamStopERP, paramStopERP);
		dJointSetSliderParam(joint, dParamStopCFM, paramStopCFM);
		v_joints.push_back(joint);
		j++;
	}
	j = 0;
	// connect front left side - left plane
	for (unsigned int i = start5; i <= start5+(n+2)*(n+2-1); i+= (n+2)){
		dJointID joint = dJointCreateSlider(world, 0);
		dJointAttach(joint, v_box.at(i), v_box.at(start4+j*n));
		dJointSetSliderAxis(joint, 0, -1, 0);
		dJointSetSliderParam(joint, dParamLoStop, loStop);
		dJointSetSliderParam(joint, dParamHiStop, hiStop);
		dJointSetSliderParam(joint, dParamBounce, paramBounce);
		dJointSetSliderParam(joint, dParamCFM, paramCFM);
		dJointSetSliderParam(joint, dParamStopERP, paramStopERP);
		dJointSetSliderParam(joint, dParamStopCFM, paramStopCFM);
		v_joints.push_back(joint);
		j++;
	}
	j = 0;
	// connect front right side - right plane
	for (unsigned int i = start5+(n+2-1); i <= start5+(n+2-1)+(n+2)*(n+2-1); i += (n+2)){
		dJointID joint = dJointCreateSlider(world, 0);
		dJointAttach(joint, v_box.at(i), v_box.at(start3+j*n));
		dJointSetSliderAxis(joint, 0, -1, 0);
		dJointSetSliderParam(joint, dParamLoStop, loStop);
		dJointSetSliderParam(joint, dParamHiStop, hiStop);
		dJointSetSliderParam(joint, dParamBounce, paramBounce);
		dJointSetSliderParam(joint, dParamCFM, paramCFM);
		dJointSetSliderParam(joint, dParamStopERP, paramStopERP);
		dJointSetSliderParam(joint, dParamStopCFM, paramStopCFM);
		v_joints.push_back(joint);
		j++;
	}
	j = 0;
	// connect back left side - left plane
	for (unsigned int i = start6; i <= start6+(n+2)*(n+2-1); i+= (n+2)){
		dJointID joint = dJointCreateSlider(world, 0);
		dJointAttach(joint, v_box.at(i), v_box.at(start4+n-1+j*n));
		dJointSetSliderAxis(joint, 0, 1, 0);
		dJointSetSliderParam(joint, dParamLoStop, loStop);
		dJointSetSliderParam(joint, dParamHiStop, hiStop);
		dJointSetSliderParam(joint, dParamBounce, paramBounce);
		dJointSetSliderParam(joint, dParamCFM, paramCFM);
		dJointSetSliderParam(joint, dParamStopERP, paramStopERP);
		dJointSetSliderParam(joint, dParamStopCFM, paramStopCFM);
		v_joints.push_back(joint);
		j++;
	}
	j = 0;
	// connect back right side - right plane
	for (unsigned int i = start6+(n+2-1); i <= start6+(n+2-1)+(n+2)*(n+2-1); i += (n+2)){
		dJointID joint = dJointCreateSlider(world, 0);
		dJointAttach(joint, v_box.at(i), v_box.at(start3+n-1+j*n));
		dJointSetSliderAxis(joint, 0, 1, 0);
		dJointSetSliderParam(joint, dParamLoStop, loStop);
		dJointSetSliderParam(joint, dParamHiStop, hiStop);
		dJointSetSliderParam(joint, dParamBounce, paramBounce);
		dJointSetSliderParam(joint, dParamCFM, paramCFM);
		dJointSetSliderParam(joint, dParamStopERP, paramStopERP);
		dJointSetSliderParam(joint, dParamStopCFM, paramStopCFM);
		v_joints.push_back(joint);
		j++;
	}

	v_boxes.push_back(v_box);

	//cout << "END_INIT_BOX\n";
} // init_box

// a cylinder
void Game::init_cylinder(){
	//cout << "INIT_CYLINDER\n";

	vector<dBodyID> v_cylinder;

	dMass m;
	dReal total_m = 0.5;
	cylinders_sphere_r = 0.3;
	double d = 2 * cylinders_sphere_r;
	cylinders_sphere_slices = 4;
	cylinders_sphere_stacks = 4;
	dMassSetSphereTotal(&m, total_m, cylinders_sphere_r);

	const int n = 20;
	const dReal center_x = mx;
	const dReal center_y = my;
	const dReal start_x = ((n-1)/2)*d + center_x;
	const dReal start_y = center_y - ((n-1)/2)*d;
	const dReal start_z = 40.0;

	dReal loStop = 0.0;
	dReal hiStop = 0.0;
	dReal paramBounce = 0.5;
	dReal paramCFM = 0.01;
	dReal paramStopERP = 0.1;
	dReal paramStopCFM = 0.001;

	unsigned int start = 0;
	start = v_cylinder.size();
	// draw a column of spheres in a circle 
	for (int i = 0; i < n; i++){ // each i
		for (int j = 0; j < n; j++){ // each column
			dBodyID b = dBodyCreate(world);
			dGeomID g = dCreateSphere(space, cylinders_sphere_r);
			dBodySetMass(b, &m);
			dGeomSetBody(g, b);
			dBodySetPosition(b, center_x + 2.0*sin(2.0*M_PI * i / (double)n), center_y + 2.0*cos(2.0*M_PI * i / (double)n), start_z + d*j);
			dBodySetData(b, NULL); // set the user pointer to something that helps us to identify the body
			v_cylinder.push_back(b);

			if(i) {
				dJointID joint1 = dJointCreateSlider(world, 0);
				dJointAttach(joint1, v_cylinder.at(start + (i-1)*n + j), v_cylinder.at(start + i*n + j));
				dJointSetSliderAxis(joint1, cos(2.0*M_PI * i / (double)n), -sin(2.0*M_PI * i / (double)n),0);		
				dJointSetSliderParam(joint1, dParamLoStop, loStop);
				dJointSetSliderParam(joint1, dParamHiStop, hiStop);
				dJointSetSliderParam(joint1, dParamBounce, paramBounce);
				dJointSetSliderParam(joint1, dParamCFM, paramCFM);
				dJointSetSliderParam(joint1, dParamStopERP, paramStopERP);
				dJointSetSliderParam(joint1, dParamStopCFM, paramStopCFM);
				v_joints.push_back(joint1);
			}
			if(j) {
				dJointID joint1 = dJointCreateSlider(world, 0);
				dJointAttach(joint1, v_cylinder.at(start + i*n + (j-1)), v_cylinder.at(start + i*n + j));
				dJointSetSliderAxis(joint1, 0, 0, 1);		
				dJointSetSliderParam(joint1, dParamLoStop, loStop);
				dJointSetSliderParam(joint1, dParamHiStop, hiStop);
				dJointSetSliderParam(joint1, dParamBounce, paramBounce);
				dJointSetSliderParam(joint1, dParamCFM, paramCFM);
				dJointSetSliderParam(joint1, dParamStopERP, paramStopERP);
				dJointSetSliderParam(joint1, dParamStopCFM, paramStopCFM);
				v_joints.push_back(joint1);
			}
			if(i==n-1) { 
				dJointID joint1 = dJointCreateSlider(world, 0);
				dJointAttach(joint1, v_cylinder.at(start + j), v_cylinder.at(start + i*n + j));
				dJointSetSliderAxis(joint1, 1, 0, 0);		
				dJointSetSliderParam(joint1, dParamLoStop, loStop);
				dJointSetSliderParam(joint1, dParamHiStop, hiStop);
				dJointSetSliderParam(joint1, dParamBounce, paramBounce);
				dJointSetSliderParam(joint1, dParamCFM, paramCFM);
				dJointSetSliderParam(joint1, dParamStopERP, paramStopERP);
				dJointSetSliderParam(joint1, dParamStopCFM, paramStopCFM);
				v_joints.push_back(joint1);
			}
		} // for each column
	} // for each i

	v_cylinders.push_back(v_cylinder);

	//cout << "END_INIT_CYLINDER\n";
} // init_cylinder

// a cloth
void Game::init_cloth(){
	//cout << "INIT_CLOTH\n";

	vector<dBodyID> v_cloth;

	dMass m;
	dReal total_m = 0.0000005;
	clothes_sphere_r = 0.6;
	double d = 2 * clothes_sphere_r;
	clothes_sphere_slices = 4;
	clothes_sphere_stacks = 4;
	dMassSetSphereTotal(&m, total_m, clothes_sphere_r);
	
	const int n = 29;
	const dReal center_x = mx;
	const dReal center_y = my;
	const dReal start_x = ((n-1)/2)*d + center_x;
	const dReal start_y = center_y - ((n-1)/2)*d;
	const dReal start_z = 40.0;

	dReal loStop = 0.0;
	dReal hiStop = 0.0;
	dReal paramBounce = 0.0;
	dReal paramCFM = 0.01;
	dReal paramStopERP = 0.5;
	dReal paramStopCFM = 0.01;

	unsigned int start = 0;
	start = v_cloth.size();
	for (int i = 0; i < n; i++){ // row
		for (int j = 0; j < n; j++){ // column
			dBodyID b = dBodyCreate(world);
			dGeomID g = dCreateSphere(space, clothes_sphere_r);
			dBodySetMass(b, &m);
			dGeomSetBody(g, b);
			dBodySetPosition(b, start_x - i * d, start_y + j * d, start_z);
			dBodySetData(b, NULL); // set the user pointer to something that helps us to identify the body
			v_cloth.push_back(b);
	
			if(i) {
				dJointID joint1 = dJointCreateSlider(world, 0);
				dJointAttach(joint1, v_cloth.at(start+(i-1)*n+j), v_cloth.at(start+i*n+j));
				dJointSetSliderAxis(joint1, -1, 0, 0);		
				dJointSetSliderParam(joint1, dParamLoStop, loStop);
				dJointSetSliderParam(joint1, dParamHiStop, hiStop);
				dJointSetSliderParam(joint1, dParamBounce, paramBounce);
				dJointSetSliderParam(joint1, dParamCFM, paramCFM);
				dJointSetSliderParam(joint1, dParamStopERP, paramStopERP);
				dJointSetSliderParam(joint1, dParamStopCFM, paramStopCFM);
				v_joints.push_back(joint1);
			}
			if(j) {
				dJointID joint1 = dJointCreateSlider(world, 0);
				dJointAttach(joint1, v_cloth.at(start+i*n+j-1), v_cloth.at(start+i*n+j));
				dJointSetSliderAxis(joint1, 0, 1, 0);		
				dJointSetSliderParam(joint1, dParamLoStop, loStop);
				dJointSetSliderParam(joint1, dParamHiStop, hiStop);
				dJointSetSliderParam(joint1, dParamBounce, paramBounce);
				dJointSetSliderParam(joint1, dParamCFM, paramCFM);
				dJointSetSliderParam(joint1, dParamStopERP, paramStopERP);
				dJointSetSliderParam(joint1, dParamStopCFM, paramStopCFM);
				v_joints.push_back(joint1);
			}
		} // for column
	} // for row

	v_clothes.push_back(v_cloth);

	//cout << "END_INIT_CLOTH\n";
} // init_cloth

// a tool
void Game::init_tool(){
	//cout << "INIT_TOOL\n";

	tool_box_lx = 2.0;
	tool_box_ly = 2.0;
	tool_box_lz = 2.0;
	tool_sphere_r = 2.0;
	tool_sphere_slices = 20;
	tool_sphere_stacks = 20;
	
	tool1 = dCreateSphere(space, tool_sphere_r);
	tool2 = dCreateSphere(space, tool_sphere_r);
	dGeomSetPosition(tool1, 0.0, -1.0, 5.0);
	dGeomSetPosition(tool2, 0.0, 1.0, 5.0);

} // init_tool

// a bean
void Game::init_bean(){
	//cout << "INIT_BEAN\n";

	dMass m;
	dReal total_m = 0.5;
	beans_sphere_r = 1.0;
	double d = 2 * beans_sphere_r;
	beans_sphere_slices = 12;
	beans_sphere_stacks = 12;
	dMassSetSphereTotal(&m, total_m, beans_sphere_r);

	const dReal* pos_tool1 = dGeomGetPosition(tool1);

	dBodyID b = dBodyCreate(world);
	dGeomID g = dCreateSphere(space, beans_sphere_r);
	dBodySetMass(b, &m);
	dGeomSetBody(g, b);
	dBodySetPosition(b, pos_tool1[0], pos_tool1[1], pos_tool1[2]-tool_sphere_r-1);
	dBodySetData(b, NULL); // set the user pointer to something that helps us to identify the body
	v_beans.push_back(b);

} // init_bean

// a bullet
void Game::init_bullet(){
	//cout << "INIT_BULLET\n";

	dMass m;
	dReal total_m = 50.0;
	bullets_sphere_r = 1.5;
	double d = 2 * bullets_sphere_r;
	bullets_sphere_slices = 15;
	bullets_sphere_stacks = 15;
	dMassSetSphereTotal(&m, total_m, bullets_sphere_r);

	const dReal* pos_tool1 = dGeomGetPosition(tool1);

	dBodyID b = dBodyCreate(world);
	dGeomID g = dCreateSphere(space, bullets_sphere_r);
	dBodySetMass(b, &m);
	dGeomSetBody(g, b);
	dBodySetPosition(b, pos_tool1[0] - tool_box_lx, pos_tool1[1], pos_tool1[2] + tool_box_lz);
	dBodySetLinearVel  (b, -40.0, 0.0, 0.0);
	dBodySetData(b, NULL); // set the user pointer to something that helps us to identify the body
	v_bullets.push_back(b);

} // init_bullet

// a sweeper
void Game::init_sweeper(){
	//cout << "INIT_SWEEPER\n";

	sweepers_cc_slices = 10;
	sweepers_cc_stacks = 10;

	const dReal* pos_tool1 = dGeomGetPosition(tool1);
	const dReal* pos_tool2 = dGeomGetPosition(tool2);

	dMass m;
	dReal total_m = 50.0;
	sweepers_cc_r = 1.0;
	sweepers_cc_l = 10; 
	int direction = 1;
	dMassSetCappedCylinderTotal (&m, total_m, direction, sweepers_cc_r, sweepers_cc_l);

	dBodyID sweeper = dBodyCreate(world);
	dGeomID g = dCreateCCylinder(space, sweepers_cc_r, sweepers_cc_l);
	dBodySetMass(sweeper, &m);
	dGeomSetBody(g, sweeper);
	dBodySetPosition(sweeper, mx - (sweepers_cc_r + 2), my-5, mz + sweepers_cc_r);
	dQuaternion q1;
	dQFromAxisAndAngle (q1, 0, 1, 1, M_PI);
	dBodySetQuaternion (sweeper, q1);
	dBodySetLinearVel  (sweeper, -100.0, 0.0, 0.0);
	dBodySetData(sweeper, NULL); // set the user pointer to something that helps us to identify the body

	v_sweepers.push_back(sweeper);

} // init_sweeper

// a sphere
void Game::init_sphere(){
	//cout << "INIT_SPHERE\n";

	spheres_r = 4.0;
	spheres_slices = 20;
	spheres_stacks = 20;
	sphere_lx = mx;
	sphere_ly = my;
	sphere_lz = mz;

	dGeomID sphere = dCreateSphere(space, spheres_r);
	dGeomSetPosition(sphere, sphere_lx, sphere_ly, sphere_lz);

	v_spheres.push_back(sphere);

} // init_sphere

// game update manager
void Game::run(){
	//cout << "RUN\n";

	SDL_Event e;

	// main loop -- one screen draw for each iteration
	while (running){

		// handle all available events before continuing (could lag?)		
		while (SDL_PollEvent(&e)){
			switch (e.type){
				case SDL_KEYDOWN:
				case SDL_KEYUP:
					handle_KeyboardEvent(e.key);
					break;
				case SDL_MOUSEMOTION:
					handle_MouseMotionEvent(e.motion);
					break;
				case SDL_MOUSEBUTTONDOWN:
				case SDL_MOUSEBUTTONUP:
					handle_MouseButtonEvent(e.button);
					break;
				case SDL_USEREVENT:
					handle_UserEvent(e.user);
					break;
				case SDL_QUIT: // generated by WM
					quit();
				// no default, ignore all other events quickly
			} // switch
		} // while SDL_PollEvent

		calc_dt();

		tick_physics();
		tick_rules();

		render();

	} // while running

} // run

// interpret key up and down event
void Game::handle_KeyboardEvent(SDL_KeyboardEvent e){
	//cout << "HANDLE_KEYBOARDEVENT\n";

	// e.keysym.sym is like SDLK_{keyname}
	// e.state is either SDL_PRESSED or SDL_RELEASED

	const dReal* pos_tool1 = dGeomGetPosition(tool1);
	const dReal* pos_tool2 = dGeomGetPosition(tool2);

	if (e.keysym.sym == SDLK_ESCAPE && e.state == SDL_PRESSED){
		quit(); // tell the main loop to finish up after next frame
	}

	if (e.keysym.sym == SDLK_r && e.state == SDL_PRESSED){
		SDL_Event r;
		r.type = SDL_USEREVENT;
		r.user.code = 0;
		SDL_PushEvent(&r);
	}

	if (e.keysym.sym == SDLK_s && e.state == SDL_PRESSED){
		shadows = !shadows; // turn on and off the shadows flag
	}

	if (e.keysym.sym == SDLK_j && e.state == SDL_PRESSED){
		joints = !joints; // turn on and off the joints flag
	}

	if (e.keysym.sym == SDLK_q && e.state == SDL_PRESSED){ 
		dGeomSetPosition(tool1, mx, pos_tool1[1]+0.5, mz);
		dGeomSetPosition(tool2, mx, pos_tool2[1]-0.5, mz);
	}
	if (e.keysym.sym == SDLK_e && e.state == SDL_PRESSED){
		dGeomSetPosition(tool1, mx, pos_tool1[1]-0.5, mz);
		dGeomSetPosition(tool2, mx, pos_tool2[1]+0.5, mz);
	}
	
	if (e.keysym.sym == SDLK_1 && e.state == SDL_PRESSED){ init_full_box(); }
	if (e.keysym.sym == SDLK_2 && e.state == SDL_PRESSED){ init_box(); }
	if (e.keysym.sym == SDLK_3 && e.state == SDL_PRESSED){ init_cylinder(); }
	if (e.keysym.sym == SDLK_4 && e.state == SDL_PRESSED){ init_cloth(); }
	if (e.keysym.sym == SDLK_5 && e.state == SDL_PRESSED){ init_sphere(); }
	if (e.keysym.sym == SDLK_6 && e.state == SDL_PRESSED){ init_full_box2(); }
	
	if (e.keysym.sym == SDLK_0 && e.state == SDL_PRESSED){ init_bean(); }

} // handle_KeyBoardEvent


// interpret mouse movement event
void Game::handle_MouseMotionEvent(SDL_MouseMotionEvent e){
	//cout << "HANDLE_MOUSEmotionEVENT\n";

	// e.x and e.y are Uint16 with window coords

	const double c = 0.025;
	const dReal* pos_tool1 = dGeomGetPosition(tool1);
	const dReal* pos_tool2 = dGeomGetPosition(tool2);

	mx += c*e.yrel;
	my += c*e.xrel;
	mz  = mz;

	dGeomSetPosition(tool1, pos_tool1[0]+c*e.yrel, pos_tool1[1]+c*e.xrel, mz);
	dGeomSetPosition(tool2, pos_tool2[0]+c*e.yrel, pos_tool2[1]+c*e.xrel, mz);

} // handle_MouseMotionEvent

// interpret mouse button up and down event
void Game::handle_MouseButtonEvent(SDL_MouseButtonEvent e){
	//cout << "HANDLE_MOUSEBUTTONEVENT\n";

	// e.button is SDL_BUTTON_LEFT, SDL_BUTTON_MIDDLE, SDL_BUTTON_RIGHT
	// e.state is either SDL_PRESSED or SDL_RELEASED

	const double c = 0.1;
	const dReal* pos_tool1 = dGeomGetPosition(tool1);
	const dReal* pos_tool2 = dGeomGetPosition(tool2);

	if (e.button == SDL_BUTTON_LEFT && e.state == SDL_PRESSED){
		init_bullet();
	}
	if (e.button == SDL_BUTTON_RIGHT && e.state == SDL_PRESSED){
		init_sweeper();
	}
	
	if (e.button == SDL_BUTTON_WHEELUP){
		mz += 0.5; 
		dGeomSetPosition(tool1, pos_tool1[0], pos_tool1[1], mz);
		dGeomSetPosition(tool2, pos_tool2[0], pos_tool2[1], mz);
	}
	if (e.button == SDL_BUTTON_WHEELDOWN){
		if (mz > 0.0){
			mz -= 0.5; 
			dGeomSetPosition(tool1, pos_tool1[0], pos_tool1[1], mz);
			dGeomSetPosition(tool2, pos_tool2[0], pos_tool2[1], mz);
		}
	}
	
} // handle_MoouseButtonEvent


// handle events generated by other game logic
// SDL has a nice event system... let's hijack it
void Game::handle_UserEvent(SDL_UserEvent e){
	//cout << "HANDLE_USEREVENT\n";

	switch(e.code) {
		case 0:
			dWorldDestroy(world);
			dJointGroupEmpty(contact_jgroup);
			mx = 0.0; my = 0.0; mz = 5.0;
			v_full_boxes.clear();
			v_full_boxes2.clear();
			v_boxes.clear();
			v_cylinders.clear();
			v_clothes.clear();
			v_beans.clear();
			v_bullets.clear();
			v_sweepers.clear();
			v_spheres.clear();
			v_joints.clear();
			init_physics();
			break;
	}

} // handle_UserEvent

// examine the wall clock time
void Game::calc_dt(){
	//cout << "CALC_DT\n";

	Uint32 now = SDL_GetTicks();
	dt = (now - ticks)/1000.0;
	ticks = now;

} //calc_dt

// tell ODE to update all physical bodies
void Game::tick_physics(){
	//cout << "TICK_PHYSICS\n";

	dJointGroupEmpty(contact_jgroup);
	dSpaceCollide(space, 0, &Game::cb_near);
    dWorldQuickStep(world, dt);

} // tick_physics

// high level game logic updater
void Game::tick_rules(){
	//cout << "TICK_RULES\n";

	// TODO: apply game rules here
	
} // tick_rules

// draw pretty stuff on the screen
void Game::render(){
	//cout << "RENDER\n";

	// reset stuff
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
    glLoadIdentity();

	// example smooth camera trick
	const dReal target[] = {mx, my, mz};
    static float ex,ey,ez;
    static float lx,ly,lz;
    static float dece = 0.0075;
    ex -= dece*(ex-target[0]);
    ey -= dece*(ey-target[1]);
    ez -= dece*(ez-target[2]);
    static float decl = 0.05;
    lx -= decl*(lx-target[0]);
    ly -= decl*(ly-target[1]);
    lz -= decl*(lz-target[2]);
	gluLookAt(ex+40,ey,ez+40, lx-15,ly,lz, 0,0,1);

	// place the light (up from here)
	const GLfloat up[] = {0.0, 0.0, 50.0, 0.0};
	glLightfv(GL_LIGHT0, GL_POSITION, up);

	if (!(v_full_boxes.empty())) render_full_boxes();
	if (!(v_full_boxes2.empty())) render_full_boxes2();
	if (!(v_boxes.empty())) render_boxes();
	if (!(v_cylinders.empty())) render_cylinders();
	if (!(v_clothes.empty())) render_clothes();
	render_tool();
	if (!(v_beans.empty())) render_beans();
	if (!(v_bullets.empty())) render_bullets();
	if (!(v_sweepers.empty())) render_sweepers();

	if (!(v_spheres.empty())) render_spheres();

	if (joints) { if (!(v_joints.empty())) render_joints(); }

	// show our pictures to the user (oh, won't they be proud)
    SDL_GL_SwapBuffers();

} // render

// render all full boxes made out of spheres
void Game::render_full_boxes(){
	//cout << "RENDER_FULL_BOXES\n";

	glPushAttrib(GL_ALL_ATTRIB_BITS);
		glColor3f(0.5, 0.5, 1.0);
		for (unsigned int j = 0; j < v_full_boxes.size(); j++) {
			vector<dBodyID> v_full_box = v_full_boxes.at(j);
			for (unsigned int i = 0; i < v_full_box.size(); i++){
				const GLfloat top[] = {0.0, 0.0, 50.0, 0.0};
				glLightfv(GL_LIGHT0, GL_POSITION, top);
				GLfloat* mat = bodyTransform(v_full_box.at(i));
			
				glPushMatrix();
					glMultMatrixf(mat);
					glutSolidSphere(full_boxes_sphere_r, full_boxes_sphere_slices, full_boxes_sphere_stacks);
				glPopMatrix();

				if (shadows){
					// place the light on the bottom for a correct reflection
					const GLfloat bottom[] = {0.0, 0.0, -50.0, 0.0};
					glLightfv(GL_LIGHT0, GL_POSITION, bottom);

					mat[14] = -mat[14]; // a reflection on the floor
					glPushMatrix();
						glMultMatrixf(mat);
						glutSolidSphere(full_boxes_sphere_r, full_boxes_sphere_slices, full_boxes_sphere_stacks);
					glPopMatrix();
				} // if shadows
			} // for some spheres
		} // for some full boxes
	glPopAttrib();

} //render_full_boxes

// render all full boxes made out of boxes
void Game::render_full_boxes2(){
	//cout << "RENDER_FULL_BOXES\n";

	glPushAttrib(GL_ALL_ATTRIB_BITS);
		glColor3f(0.5, 0.5, 1.0);
		for (unsigned int j = 0; j < v_full_boxes2.size(); j++) {
			vector<dBodyID> v_full_box2 = v_full_boxes2.at(j);
			for (unsigned int i = 0; i < v_full_box2.size(); i++){
				const GLfloat top[] = {0.0, 0.0, 50.0, 0.0};
				glLightfv(GL_LIGHT0, GL_POSITION, top);
				GLfloat* mat = bodyTransform(v_full_box2.at(i));
			
				glPushMatrix();
					glMultMatrixf(mat);
					glutSolidCube(full_boxes2_box_lx);
				glPopMatrix();

				if (shadows){
					// place the light on the bottom for a correct reflection
					const GLfloat bottom[] = {0.0, 0.0, -50.0, 0.0};
					glLightfv(GL_LIGHT0, GL_POSITION, bottom);

					mat[14] = -mat[14]; // a reflection on the floor
					glPushMatrix();
						glMultMatrixf(mat);
						glutSolidCube(full_boxes2_box_lx);
					glPopMatrix();
				} // if shadows
			} // for some spheres
		} // for some full boxes
	glPopAttrib();

} //render_full_boxes2

// render all boxes
void Game::render_boxes(){
	//cout << "RENDER_BOXES\n";
	
	glPushAttrib(GL_ALL_ATTRIB_BITS);
		glColor3f(0.5, 1.0, 0.75);
		for (unsigned int j = 0; j < v_boxes.size(); j++){
			vector<dBodyID> v_box = v_boxes.at(j);
			for (unsigned int i = 0; i < v_box.size(); i++){
				const GLfloat top[] = {0.0, 0.0, 50.0, 0.0};
				glLightfv(GL_LIGHT0, GL_POSITION, top);

				GLfloat* mat = bodyTransform(v_box.at(i));
				glPushMatrix();
					glMultMatrixf(mat);
					glutSolidSphere(boxes_sphere_r, boxes_sphere_slices, boxes_sphere_stacks);
				glPopMatrix();

				if (shadows){
					// place the light on the bottom for a correct reflection
					const GLfloat bottom[] = {0.0, 0.0, -50.0, 0.0};
					glLightfv(GL_LIGHT0, GL_POSITION, bottom);

					mat[14] = -mat[14]; // a reflection on the floor
					glPushMatrix();
						glMultMatrixf(mat);
						glutSolidSphere(boxes_sphere_r, boxes_sphere_slices, boxes_sphere_stacks);
					glPopMatrix();
				} // if shadows
			} // for some spheres
		} // for some boxes
	glPopAttrib();

} // render_boxes

// render all cylinders
void Game::render_cylinders(){
	//cout << "RENDER_CYLINDERS\n";

	glPushAttrib(GL_ALL_ATTRIB_BITS);
		glColor3f(1.0, 1.0, 0.5);
		for (unsigned int j = 0; j < v_cylinders.size(); j++){
			vector<dBodyID> v_cylinder = v_cylinders.at(j);
			for (unsigned int i = 0; i < v_cylinder.size(); i++){
				const GLfloat top[] = {0.0, 0.0, 1.0, 0.0};
				glLightfv(GL_LIGHT0, GL_POSITION, top);

				GLfloat* mat = bodyTransform(v_cylinder.at(i));
				glPushMatrix();
					glMultMatrixf(mat);
					glutSolidSphere(cylinders_sphere_r, cylinders_sphere_slices, cylinders_sphere_stacks);
				glPopMatrix();

				if (shadows){
					// place the light on the bottom for a correct reflection
					const GLfloat bottom[] = {0.0, 0.0, -1.0, 0.0};
					glLightfv(GL_LIGHT0, GL_POSITION, bottom);

					mat[14] = -mat[14]; // a reflection on the floor
					glPushMatrix();
						glMultMatrixf(mat);
						glutSolidSphere(cylinders_sphere_r, cylinders_sphere_slices, cylinders_sphere_stacks);
					glPopMatrix();
				} // if shadows
			} // for some spheres
		} // for some cylinders
	glPopAttrib();
	
} // render_cylinders

// render all clothes
void Game::render_clothes(){
	//cout << "RENDER_CLOTHES\n";

	glPushAttrib(GL_ALL_ATTRIB_BITS);
		glColor3f(1.0, 0.5, 1.0);
		for (unsigned int j = 0; j < v_clothes.size(); j++){
			vector<dBodyID> v_cloth = v_clothes.at(j);
			for (unsigned int i = 0; i < v_cloth.size(); i++){
				const GLfloat top[] = {0.0, 0.0, 1.0, 0.0};
				glLightfv(GL_LIGHT0, GL_POSITION, top);

				GLfloat* mat = bodyTransform(v_cloth.at(i));
				glPushMatrix();
					glMultMatrixf(mat);
					glutSolidSphere(clothes_sphere_r, clothes_sphere_slices, clothes_sphere_stacks);
				glPopMatrix();

				if (shadows){
					// place the light on the bottom for a correct reflection
					const GLfloat bottom[] = {0.0, 0.0, -1.0, 0.0};
					glLightfv(GL_LIGHT0, GL_POSITION, bottom);

					mat[14] = -mat[14]; // a reflection on the floor
					glPushMatrix();
						glMultMatrixf(mat);
						glutSolidSphere(clothes_sphere_r, clothes_sphere_slices, clothes_sphere_stacks);
					glPopMatrix();
				} // if shadows
			} // for some spheres
		} // for some clothes
	glPopAttrib();

} // render_clothes

// render a tool
void Game::render_tool(){
	//cout << "RENDER_TOOL\n";

	glPushAttrib(GL_ALL_ATTRIB_BITS);
		const GLfloat top[] = {0.0, 0.0, 50.0, 0.0};
		glLightfv(GL_LIGHT0, GL_POSITION, top);

		GLfloat* mat1 = geomTransform(tool1);
		glColor3f(1.0, 1.0, 0.0);
		glPushMatrix();
			glMultMatrixf(mat1);
			glutSolidSphere(tool_sphere_r, tool_sphere_slices, tool_sphere_stacks);
		glPopMatrix();

		GLfloat* mat2 = geomTransform(tool2);
		glColor3f(0.75, 0.5, 0.0);
		glPushMatrix();
			glMultMatrixf(mat2);
			glutSolidSphere(tool_sphere_r, tool_sphere_slices, tool_sphere_stacks);
		glPopMatrix();

		if (shadows){
			// place the light on the bottom for a correct reflection
			const GLfloat bottom[] = {0.0, 0.0, -50.0, 0.0};
			glLightfv(GL_LIGHT0, GL_POSITION, bottom);

			mat1 = geomTransform(tool1);
			mat1[14] = -mat1[14]; // a reflection on the floor
			glColor3f(0.75, 0.75, 0.0);
			glPushMatrix();
				glMultMatrixf(mat1);
				glutSolidSphere(tool_sphere_r, tool_sphere_slices, tool_sphere_stacks);
			glPopMatrix();

			mat2 = geomTransform(tool2);
			mat2[14] = -mat2[14]; // a reflection on the floor
			glColor3f(0.5, 0.25, 0.0);
			glPushMatrix();
				glMultMatrixf(mat2);
				glutSolidSphere(tool_sphere_r, tool_sphere_slices, tool_sphere_stacks);
			glPopMatrix();
		} // if shadows
    glPopAttrib();

} // render_tool

// render all beans
void Game::render_beans(){
	//cout << "RENDER_BEAN\n";

	glPushAttrib(GL_ALL_ATTRIB_BITS);
		glColor3f(0.75, r(0.5, 0.25), 0.25);
		for (unsigned int i = 0; i < v_beans.size(); i++){
			GLfloat* mat = bodyTransform(v_beans.at(i));
			const GLfloat top[] = {0.0, 0.0, 50.0, 0.0};
			glLightfv(GL_LIGHT0, GL_POSITION, top);
			glPushMatrix();
				glMultMatrixf(mat);
				glutSolidSphere(beans_sphere_r, beans_sphere_slices, beans_sphere_stacks);
			glPopMatrix();

			if (shadows){
				// place the light on the bottom for a correct reflection
				const GLfloat bottom[] = {0.0, 0.0, -50.0, 0.0};
				glLightfv(GL_LIGHT0, GL_POSITION, bottom);
				mat[14] = -mat[14];
				glPushMatrix();
					glMultMatrixf(mat);
					glutSolidSphere(beans_sphere_r, beans_sphere_slices, beans_sphere_stacks);
				glPopMatrix();
			} // if shadows
		} // for some beans
	glPopAttrib();

} // render_beans

// render all bullets
void Game::render_bullets(){
	//cout << "RENDER_BULLETS\n";
	
	glPushAttrib(GL_ALL_ATTRIB_BITS);
		glColor3f(1.0, 0.75, 0.25);
		for (unsigned int i = 0; i < v_bullets.size(); i++){
			const GLfloat top[] = {0.0, 0.0, 50.0, 0.0};
			glLightfv(GL_LIGHT0, GL_POSITION, top);

			GLfloat* mat = bodyTransform(v_bullets.at(i));
			glPushMatrix();
				glMultMatrixf(mat);
				glutSolidSphere(bullets_sphere_r, bullets_sphere_slices, bullets_sphere_stacks);
			glPopMatrix();

			if (shadows){
				// place the light on the bottom for a correct reflection
				const GLfloat bottom[] = {0.0, 0.0, -50.0, 0.0};
				glLightfv(GL_LIGHT0, GL_POSITION, bottom);
				mat[14] = -mat[14];
				glPushMatrix();
					glMultMatrixf(mat);
					glutSolidSphere(bullets_sphere_r, bullets_sphere_slices, bullets_sphere_stacks);
				glPopMatrix();
			} // if shadows
		} // for some bullets
	glPopAttrib();

} // render_bullets

// render all sweepers
void Game::render_sweepers(){
	//cout << "RENDER_SWEEPERS\n";

	glPushAttrib(GL_ALL_ATTRIB_BITS);
		glColor3f(1.0, 0.75, 0.25);
		for (unsigned int i = 0; i < v_sweepers.size(); i++){
			const GLfloat top[] = {0.0, 0.0, 50.0, 0.0};
			glLightfv(GL_LIGHT0, GL_POSITION, top);

			GLfloat* mat = bodyTransform(v_sweepers.at(i));
			glPushMatrix();
				glMultMatrixf(mat);
				gluCylinder(quad, sweepers_cc_r, sweepers_cc_r, sweepers_cc_l, sweepers_cc_slices, sweepers_cc_stacks);
			glPopMatrix();

			if (shadows){
				// place the light on the bottom for a correct reflection
				const GLfloat bottom[] = {0.0, 0.0, -50.0, 0.0};
				glLightfv(GL_LIGHT0, GL_POSITION, bottom);
				mat[14] = -mat[14];
				glPushMatrix();
					glMultMatrixf(mat);
					gluCylinder(quad, sweepers_cc_r, sweepers_cc_r, sweepers_cc_l, sweepers_cc_slices, sweepers_cc_stacks);
				glPopMatrix();
			} // if shadows
		} // for some bullets
	glPopAttrib();

} // render_sweepers

// render all joints
void Game::render_joints(){
	//cout << "RENDER_JOINTS\n";

	glPushAttrib(GL_ALL_ATTRIB_BITS);
	glColor3f(1.0, 1.0, 1.0);
	glBegin(GL_LINES);
		for(unsigned int i = 0; i < v_joints.size(); i++) {
			glVertex3f(dBodyGetPosition(dJointGetBody(v_joints[i],0))[0],dBodyGetPosition(dJointGetBody(v_joints[i],0))[1],dBodyGetPosition(dJointGetBody(v_joints[i],0))[2]);
			glVertex3f(dBodyGetPosition(dJointGetBody(v_joints[i],1))[0],dBodyGetPosition(dJointGetBody(v_joints[i],1))[1],dBodyGetPosition(dJointGetBody(v_joints[i],1))[2]);
		}
	glEnd();
	glPopAttrib();
	
} // render_joints

// render all spheres
void Game::render_spheres(){
	//cout << "RENDER_SPHERES\n";

	glPushAttrib(GL_ALL_ATTRIB_BITS);
		glColor3f(0.0, 0.0, 0.8);
		for (unsigned int i = 0; i < v_spheres.size(); i++){
			const GLfloat top[] = {0.0, 0.0, 50.0, 0.0};
			glLightfv(GL_LIGHT0, GL_POSITION, top);

			GLfloat* mat = geomTransform(v_spheres.at(i));
			glPushMatrix();
				glMultMatrixf(mat);
				glutSolidSphere(spheres_r, spheres_slices, spheres_stacks);
			glPopMatrix();
		
			if (shadows){
				// place the light on the bottom for a correct reflection
				const GLfloat bottom[] = {0.0, 0.0, -50.0, 0.0};
				glLightfv(GL_LIGHT0, GL_POSITION, bottom);
		
				mat[14] = -mat[14]; // a reflection on the floor
				glPushMatrix();
					glMultMatrixf(mat);
					glutSolidSphere(spheres_r, spheres_slices, spheres_stacks);
				glPopMatrix();
			} // if shadows
		} // for all spheres
	glPopAttrib();

} // render_spheres

// build GL matrix from given body position/rotation
// return pointer to local buffer, modified by subsequent calls
GLfloat*  Game::bodyTransform(dBodyID b){
	//cout << "BODYTRANSFORM\n";

	static GLfloat mat[16]; // to be returned

	const dReal* pos = dBodyGetPosition(b);
    const dReal* rot = dBodyGetRotation(b);
   
	// 0 applies no perspective transform
	// 1 preserves homogeneous coordinate
    mat[0] = rot[0];
    mat[1] = rot[4];
    mat[2] = rot[8];
    mat[3] = 0;
    mat[4] = rot[1];
    mat[5] = rot[5];
    mat[6] = rot[9];
    mat[7] = 0;
    mat[8] = rot[2];
    mat[9] = rot[6];
    mat[10] = rot[10];
    mat[11] = 0;
    mat[12] = pos[0];
    mat[13] = pos[1];
    mat[14] = pos[2];
    mat[15] = 1;

	return mat;
} // bodyTransform

// build GL matrix from given body position/rotation
// return pointer to local buffer, modified by subsequent calls
GLfloat* Game::geomTransform(dGeomID b){
	//cout << "GEOMTRANSFORM\n";

	static GLfloat mat[16]; // to be returned

	const dReal* pos = dGeomGetPosition(b);
    const dReal* rot = dGeomGetRotation(b);
   
	// 0 applies no perspective transform
	// 1 preserves homogeneous coordinate
    mat[0] = rot[0];
    mat[1] = rot[4];
    mat[2] = rot[8];
    mat[3] = 0;
    mat[4] = rot[1];
    mat[5] = rot[5];
    mat[6] = rot[9];
    mat[7] = 0;
    mat[8] = rot[2];
    mat[9] = rot[6];
    mat[10] = rot[10];
    mat[11] = 0;
    mat[12] = pos[0];
    mat[13] = pos[1];
    mat[14] = pos[2];
    mat[15] = 1;

	return mat;
} // geomTransform

// handle collisions between physical bodies (a complex function!)
void Game::cb_near(void *data, dGeomID o1, dGeomID o2){
	//cout << "CB_NEAR\n";

    if (dGeomIsSpace (o1) || dGeomIsSpace (o2)){

      // colliding a space with something
      dSpaceCollide2 (o1,o2,data,&Game::cb_near);
      // collide all geoms internal to the space(s)
      if (dGeomIsSpace (o1)) dSpaceCollide ((dSpaceID)o1,data,&Game::cb_near);
      if (dGeomIsSpace (o2)) dSpaceCollide ((dSpaceID)o2,data,&Game::cb_near);

    } else{

		// colliding two non-space geoms, so generate contact
		// points between o1 and o2
        dBodyID b1 = dGeomGetBody(o1);
		dBodyID b2 = dGeomGetBody(o2);

		dContact contacts[g->max_contacts];
		int n = dCollide(o1,o2, g->max_contacts, &contacts[0].geom, sizeof(dContact));
		for (int i = 0; i < n; i++){
			contacts[i].surface.mode = 0; // | more
			contacts[i].surface.mu = 1000;
			dJointID c = dJointCreateContact(g->world,g->contact_jgroup,&contacts[i]);
			dJointAttach(c,b1,b2);
		} // for n
    }

} // cb_near

// THE END