#ifndef __GAME_H__
#define __GAME_H__

#include <cstdlib>
#include <iostream>
#include <vector>
using namespace std;

#include <SDL.h>
#include <SDL_image.h>
#include <SDL_mixer.h>
#include <ode/ode.h>

#ifdef __APPLE__
#	include <GLUT/glut.h>
#else
#	include <glut.h>
#endif

// "singleton" instance
class Game;
Game* g;

// a high-level container class for game states and logic
class Game {

// in a serious game, much of these would be marked private
public:

	/////////////
	// METHODS //
	/////////////

	Game(int argc, char** argv)
		: argc(argc),argv(argv),win_width(1024),win_height(768),show_fullscreen(true),dt(0),running(true),return_code(0),mx(0.0),my(0.0),mz(5.0),shadows(true),joints(false)
		{
			g = this;
			this->process_args();
		};

	void process_args(); // examine command line arguments

	void init();           // call init_*
	void init_rules();     // high level game setup
	void init_io();        // SDL setup
	void init_gfx();       // GL setup, visual assets preparation (using SDL_image)
	void init_snd();       // sound setup, audio assets preparation (using SDL_mixer)
	void init_physics();   // ODE setup, basic world setup
	void init_full_box();  // full box setup made out of spheres
	void init_full_box2(); // full box setup made out of boxes
	void init_box();       // box setup
	void init_cylinder();  // cylinder setup
	void init_cloth();     // cloth setup
	void init_tool();      // tool setup
	void init_bean();      // bean setup
	void init_bullet();    // bullet setup
	void init_sweeper();   // sweeper setup
	void init_sphere();    // sphere setup

	void run(); // main event loop (handle, tick, draw)

	// handle common SDL events
	void handle_KeyboardEvent(SDL_KeyboardEvent);
	void handle_MouseMotionEvent(SDL_MouseMotionEvent);
	void handle_MouseButtonEvent(SDL_MouseButtonEvent);
	void handle_UserEvent(SDL_UserEvent); // we borrow SDL's event system

	void calc_dt(); // update ticks and dt using SDL_GetTicks

	void tick_physics(); // ODE world step
	void tick_rules();   // per frame logical rules update

	void render();             // fill screen with fun stuff
	void render_full_boxes();  // render all full boxes made out of spheres
	void render_full_boxes2(); // render all full boxes made out of boxes
	void render_boxes();       // render all boxes
	void render_cylinders();   // render all cylinders
	void render_clothes();     // render all clothes
	void render_tool();        // render a tool
	void render_beans();       // render all beans
	void render_bullets();     // render all bullets
	void render_sweepers();    // render all sweepers
	void render_joints();      // render all joints
	void render_spheres();     // render all spheres

	static void cb_near(void*, dGeomID, dGeomID);

	// utility functions

	static double r(double max = 1.0, double min = 0.0){
		return (((max - min) * (double)rand() / (double)RAND_MAX) + min);
	}
	double ar() {return (double)win_width / (double)win_height;}
	void quit(int code = 0) {running = false; return_code = code;}
	
	static GLfloat* bodyTransform(dBodyID);
	static GLfloat* geomTransform(dGeomID);

	////////////////
	// PROPERTIES //
	////////////////

	// MISC
	int argc;
	char **argv;
	Uint32 ticks; // last result of SDL_GetTicks
	double dt; // time in seconds since last update
	inline double FPS() {return dt;}                   // frames per second
	inline double SPF() {return 1.0 / dt;}             // seconds per frame
	inline double T() {return (double)ticks / 1000.0;} // seconds per frame
	bool running;      // flag for event loop
	bool shadows;      // flag for showing shadows/reflections
	bool joints;       // flag for showing joints
	int return_code; // value to return to OS from main
	dReal mx, my, mz; // mouse coordinate

	// RULES
	// TODO: add some game specific stuff here!

	// IO
	GLsizei win_width;
	GLsizei win_height;
	bool show_fullscreen;

	// GRAPHICS
	GLuint tex_something;
	GLuint tex_mask;
	GLUquadric* quad;

	// SOUND
	Mix_Music* music_ambient;
	Mix_Chunk* chunk_rattle;

	// PHYSICS
	dWorldID world;
	dSpaceID space;

	vector<vector<dBodyID> > v_full_boxes;
	dReal full_boxes_sphere_r;
	GLint full_boxes_sphere_slices;
	GLint full_boxes_sphere_stacks;
	
	vector<vector<dBodyID> > v_full_boxes2;
	dReal full_boxes2_box_lx;
	dReal full_boxes2_box_ly;
	dReal full_boxes2_box_lz;

	vector<vector<dBodyID> > v_boxes;
	dReal boxes_sphere_r;
	GLint boxes_sphere_slices;
	GLint boxes_sphere_stacks;

	vector <vector<dBodyID> > v_cylinders;
	dReal cylinders_sphere_r;
	GLint cylinders_sphere_slices;
	GLint cylinders_sphere_stacks;

	vector<vector<dBodyID> > v_clothes;
	dReal clothes_sphere_r;
	GLint clothes_sphere_slices;
	GLint clothes_sphere_stacks;

	dGeomID tool1;
	dGeomID tool2;
	dReal tool_box_lx;
	dReal tool_box_ly;
	dReal tool_box_lz;
	dReal tool_sphere_r;
	GLint tool_sphere_slices;
	GLint tool_sphere_stacks;

	vector<dBodyID> v_beans;
	dReal beans_sphere_r;
	GLint beans_sphere_slices;
	GLint beans_sphere_stacks;

	vector<dBodyID> v_bullets;
	dReal bullets_sphere_r;
	GLint bullets_sphere_slices;
	GLint bullets_sphere_stacks;

	vector<dBodyID> v_sweepers;
	dReal sweepers_cc_r;
	dReal sweepers_cc_l;
	GLint sweepers_cc_slices;
	GLint sweepers_cc_stacks;

	vector<dGeomID> v_spheres;
	dReal spheres_r;
	GLint spheres_slices;
	GLint spheres_stacks;
	dReal sphere_lx;
	dReal sphere_ly;
	dReal sphere_lz;

	vector<dJointID> v_joints;
	dJointGroupID contact_jgroup;

	static const int max_contacts = 8;

}; // class Game

#endif